import pytest

from geopy import exc
from geopy.geocoders import Geocodio
from geopy.point import Point
from test.geocoders.util import BaseTestGeocoder, env


class TestGeocodio(BaseTestGeocoder):

    @classmethod
    def make_geocoder(cls, **kwargs):
        return Geocodio(api_key=env['GEOCODIO_KEY'], **kwargs)

    async def test_user_agent_custom(self):
        geocoder = self.make_geocoder(user_agent='my_user_agent/1.0')
        assert geocoder.headers['User-Agent'] == 'my_user_agent/1.0'

    async def test_error_with_only_street(self):
        with pytest.raises(exc.GeocoderQueryError):
            await self.geocode_run(
                {'query': {'street': '435 north michigan ave'}},
                {},
            )

    async def test_geocode(self):
        await self.geocode_run(
            {"query": "435 north michigan ave, chicago il 60611 usa"},
            {"latitude": 41.89037, "longitude": -87.623192},
        )

    async def test_geocode_from_components(self):
        await self.geocode_run(
            {
                "query": {
                    "street": "435 north michigan ave",
                    "city": "chicago",
                    "state": "IL",
                    "postal_code": "60611"
                },
            },
            {"latitude": 41.89037, "longitude": -87.623192},
        )

    async def test_geocode_many_results(self):
        result = await self.geocode_run(
            {"query": "Springfield", "exactly_one": False},
            {}
        )
        assert len(result) > 1

    async def test_reverse(self):
        location = await self.reverse_run(
            {"query": Point(40.75376406311989, -73.98489005863667)},
            {"latitude": 40.75376406311989, "longitude": -73.98489005863667},
        )
        assert "new york" in location.address.lower()

    async def test_geocode_no_result(self):
        await self.geocode_run(
            {"query": "dksajdkjashdkjashdjasghd"},
            {},
            expect_failure=True,
        )

    async def test_geocode_structured_no_result(self):
        await self.geocode_run(
            {"query": {"city": "dkasdjksahdksajhd", "street": "sdahaskjdhask"}},
            {},
            expect_failure=True,
        )

    async def test_reverse_no_result(self):
        await self.reverse_run(
            # North Atlantic Ocean
            {"query": (35.173809, -37.485351)},
            {},
            expect_failure=True,
        )



import pytest

from geopy.geocoders import Geokeo
from geopy.point import Point
from test.geocoders.util import BaseTestGeocoder, env


class TestUnitGeokeo:

    def test_user_agent_custom(self):
        geocoder = Geokeo(
            api_key='DUMMYKEY1234',
            user_agent='my_user_agent/1.0'
        )
        assert geocoder.headers['User-Agent'] == 'my_user_agent/1.0'


class TestGeokeo(BaseTestGeocoder):

    @classmethod
    def make_geocoder(cls, **kwargs):
        return Geokeo(
            api_key=env['GEOKEY_KEY'],
            timeout=10,
            **kwargs
        )

    async def test_geocode(self):
        await self.geocode_run(
            {"query": "empire state building"},
            {"latitude": 40.74843124430164, "longitude": -73.9856567114413},
        )

    async def test_geocode_country(self):
        await self.geocode_run(
            {"query": "moscow", "country": "us"},
            {"latitude": 46.730685699940594, "longitude": -116.99875169095164},
        )
        await self.geocode_run(
            {"query": "moscow", "country": "ru"},
            {"latitude": 55.75044609968516, "longitude": 37.6174943},
        )

    @pytest.mark.xfail(reason="invalid json is returned at the moment")
    async def test_geocode_empty_result(self):
        await self.geocode_run(
            {"query": "xqj37"},
            {},
            expect_failure=True
        )

    async def test_reverse(self):
        location = await self.reverse_run(
            {"query": Point(40.74843124430164, -73.9856567114413)},
            {"latitude": 40.74843124430164, "longitude": -73.9856567114413},
        )
        assert "Empire State Building" in location.address

    async def test_reverse_empty_result(self):
        await self.reverse_run(
            {"query": Point(0.05, -0.15)},
            {},
            expect_failure=True,
        )



import contextlib
import json
import os
from abc import ABC, abstractmethod
from unittest.mock import ANY, patch

import pytest

from geopy import exc
from geopy.adapters import BaseAsyncAdapter
from geopy.location import Location

_env = {}
try:
    with open(".test_keys") as fp:
        _env.update(json.loads(fp.read()))
except OSError:
    _env.update(os.environ)


class SkipIfMissingEnv(dict):
    def __init__(self, env):
        super().__init__(env)
        self.is_internet_access_allowed = None

    def __getitem__(self, key):
        assert self.is_internet_access_allowed is not None
        if key not in self or not super().__getitem__(key):
            if self.is_internet_access_allowed:
                pytest.skip("Missing geocoder credential: %s" % (key,))
            else:
                # Generate some dummy token. We won't perform a networking
                # request anyways.
                return "dummy"
        return super().__getitem__(key)


env = SkipIfMissingEnv(_env)


class BaseTestGeocoder(ABC):
    """
    Base for geocoder-specific test cases.
    """

    geocoder = None
    delta = 0.5

    @pytest.fixture(scope='class', autouse=True)
    async def class_geocoder(_, request, patch_adapter, is_internet_access_allowed):
        """Prepare a class-level Geocoder instance."""
        cls = request.cls
        env.is_internet_access_allowed = is_internet_access_allowed

        geocoder = cls.make_geocoder()
        cls.geocoder = geocoder

        run_async = isinstance(geocoder.adapter, BaseAsyncAdapter)
        if run_async:
            async with geocoder:
                yield geocoder
        else:
            yield geocoder

    @classmethod
    @contextlib.asynccontextmanager
    async def inject_geocoder(cls, geocoder):
        """An async context manager allowing to inject a custom
        geocoder instance in a single test method which will
        be used by the `geocode_run`/`reverse_run` methods.
        """
        with patch.object(cls, 'geocoder', geocoder):
            run_async = isinstance(geocoder.adapter, BaseAsyncAdapter)
            if run_async:
                async with geocoder:
                    yield geocoder
            else:
                yield geocoder

    @pytest.fixture(autouse=True)
    def ensure_no_geocoder_assignment(self):
        yield
        assert self.geocoder is type(self).geocoder, (
            "Detected `self.geocoder` assignment. "
            "Please use `async with inject_geocoder(my_geocoder):` "
            "instead, which supports async adapters."
        )

    @classmethod
    @abstractmethod
    def make_geocoder(cls, **kwargs):  # pragma: no cover
        pass

    async def geocode_run(
        self, payload, expected,
        *,
        skiptest_on_errors=True,
        expect_failure=False
    ):
        """
        Calls geocoder.geocode(**payload), then checks against `expected`.
        """
        cls = type(self)
        result = await self._make_request(
            self.geocoder, 'geocode',
            skiptest_on_errors=skiptest_on_errors,
            **payload,
        )
        if expect_failure:
            assert result is None
            return
        if result is None:
            pytest.fail('%s: No result found' % cls.__name__)
        if result == []:
            pytest.fail('%s returned an empty list instead of None' % cls.__name__)
        self._verify_request(result, exactly_one=payload.get('exactly_one', True),
                             **expected)
        return result

    async def reverse_run(
        self, payload, expected,
        *,
        skiptest_on_errors=True,
        expect_failure=False
    ):
        """
        Calls geocoder.reverse(**payload), then checks against `expected`.
        """
        cls = type(self)
        result = await self._make_request(
            self.geocoder, 'reverse',
            skiptest_on_errors=skiptest_on_errors,
            **payload,
        )
        if expect_failure:
            assert result is None
            return
        if result is None:
            pytest.fail('%s: No result found' % cls.__name__)
        if result == []:
            pytest.fail('%s returned an empty list instead of None' % cls.__name__)
        self._verify_request(result, exactly_one=payload.get('exactly_one', True),
                             **expected)
        return result

    async def reverse_timezone_run(self, payload, expected, *, skiptest_on_errors=True):
        timezone = await self._make_request(
            self.geocoder, 'reverse_timezone',
            skiptest_on_errors=skiptest_on_errors,
            **payload,
        )
        if expected is None:
            assert timezone is None
        else:
            assert timezone.pytz_timezone == expected

        return timezone

    async def _make_request(self, geocoder, method, *, skiptest_on_errors, **kwargs):
        cls = type(self)
        call = getattr(geocoder, method)
        run_async = isinstance(geocoder.adapter, BaseAsyncAdapter)
        try:
            if run_async:
                result = await call(**kwargs)
            else:
                result = call(**kwargs)
        except exc.GeocoderRateLimited as e:
            if not skiptest_on_errors:
                raise
            pytest.skip(
                "%s: Rate-limited, retry-after %s" % (cls.__name__, e.retry_after)
            )
        except exc.GeocoderQuotaExceeded:
            if not skiptest_on_errors:
                raise
            pytest.skip("%s: Quota exceeded" % cls.__name__)
        except exc.GeocoderTimedOut:
            if not skiptest_on_errors:
                raise
            pytest.skip("%s: Service timed out" % cls.__name__)
        except exc.GeocoderUnavailable:
            if not skiptest_on_errors:
                raise
            pytest.skip("%s: Service unavailable" % cls.__name__)
        return result

    def _verify_request(
            self,
            result,
            latitude=ANY,
            longitude=ANY,
            address=ANY,
            exactly_one=True,
            delta=None,
    ):
        if exactly_one:
            assert isinstance(result, Location)
        else:
            assert isinstance(result, list)

        item = result if exactly_one else result[0]
        delta = delta or self.delta

        expected = (
            pytest.approx(latitude, abs=delta) if latitude is not ANY else ANY,
            pytest.approx(longitude, abs=delta) if longitude is not ANY else ANY,
            address,
        )
        received = (
            item.latitude,
            item.longitude,
            item.address,
        )
        assert received == expected



#!/usr/bin/env python

from setuptools import setup

from geopy import __version__ as version

EXTRAS_DEV_LINT = [
    "flake8>=5.0,<5.1",
    "isort>=5.10.0,<5.11.0",
]

EXTRAS_DEV_TEST = [
    "coverage",
    "pytest-asyncio>=0.17",
    "pytest>=3.10",
    "sphinx<=4.3.2",  # `docutils` from sphinx is used in tests
]

EXTRAS_DEV_DOCS = [
    "readme_renderer",
    "sphinx<=4.3.2",
    "sphinx-issues",
    "sphinx_rtd_theme>=0.5.0",
]

setup(
    download_url=(
        'https://github.com/geopy/geopy/archive/%s.tar.gz' % version
    ),
    extras_require={
        "dev": sorted(set(
            EXTRAS_DEV_LINT +
            EXTRAS_DEV_TEST +
            EXTRAS_DEV_DOCS
        )),
        "dev-lint": EXTRAS_DEV_LINT,
        "dev-test": EXTRAS_DEV_TEST,
        "dev-docs": EXTRAS_DEV_DOCS,
        "aiohttp": ["aiohttp"],
        "requests": [
            "urllib3>=1.24.2",
            # ^^^ earlier versions would work, but a custom ssl
            # context would silently have system certificates be loaded as
            # trusted: https://github.com/urllib3/urllib3/pull/1566

            "requests>=2.16.2",
            # ^^^ earlier versions would work, but they use an older
            # vendored version of urllib3 (see note above)
        ],
        "timezone": ["pytz"],
    },
)
